/* Copyright (C) 2010-2020 Frankfurt Institute for Advanced Studies, Goethe-Universität Frankfurt, Frankfurt
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Valentina Akishina, Igor Kulakov [committer], Maksym Zyzak */

#ifndef CaGridEntry_h
#define CaGridEntry_h

#include "CaConstants.h"
#include "CaHit.h"
#include "CaSimd.h"

namespace cbm::algo::ca
{

  /// \brief A class to store hit information on the ca::Grid
  ///
  struct GridEntry {

    GridEntry() = default;

    GridEntry(const ca::Hit& hit) { Set(hit); };

    void Set(const ca::Hit& hit)
    {
      x      = hit.x;
      y      = hit.y;
      z      = hit.z;
      t      = hit.t;
      dx2    = hit.dx2;
      dxy    = hit.dxy;
      dy2    = hit.dy2;
      dt2    = hit.dt2;
      rangeX = hit.rangeX;
      rangeY = hit.rangeY;
      rangeT = hit.rangeT;

      fIsSuppressed = 0;
    }

    fscal Z() const { return z; }
    fscal T() const { return t; }
    fscal X() const { return x; }
    fscal Y() const { return y; }
    fscal dX2() const { return dx2; }
    fscal dXY() const { return dxy; }
    fscal dY2() const { return dy2; }
    fscal dT2() const { return dt2; }
    fscal RangeX() const { return rangeX; }
    fscal RangeY() const { return rangeY; }
    fscal RangeT() const { return rangeT; }

    bool IsSuppressed() const { return fIsSuppressed; }

    void SetIsSuppresed(bool val) { fIsSuppressed = val; }

  private:
    constexpr static fscal kUndef = 0.;  // TODO: test with constants::Undef<fscal>

    fscal x {kUndef};       // x coordinate of the hit
    fscal y {kUndef};       // y coordinate of the hit
    fscal z {kUndef};       // z coordinate of the hit
    fscal t {kUndef};       // t coordinate of the hit
    fscal dx2 {kUndef};     // x error^2 of the hit
    fscal dxy {kUndef};     // xy covariance of the hit
    fscal dy2 {kUndef};     // y error^2 of the hit
    fscal dt2 {kUndef};     // t error^2 of the hit
    fscal rangeX {kUndef};  // x range of the hit
    fscal rangeY {kUndef};  // y range of the hit
    fscal rangeT {kUndef};  // t range of the hit

    /// fIsSuppressed flag is used to suppress duplicated hits at the module overlaps
    // TODO: collect those hits on the track instead of suppressing them
    bool fIsSuppressed {0};  // is the hit suppressed by another hit
  };

}  // namespace cbm::algo::ca

#endif
