/* Copyright (C) 2023 FIAS Frankfurt Institute for Advanced Studies, Frankfurt / Main
   SPDX-License-Identifier: GPL-3.0-only
   Authors: Felix Weiglhofer [committer] */
#ifndef CBM_ALGO_GLOBAL_RECO_H
#define CBM_ALGO_GLOBAL_RECO_H

#include <xpu/host.h>

#include "EventbuildChain.h"
#include "SubChain.h"
#include "UnpackChain.h"
#include "sts/HitfinderChain.h"

namespace fles
{
  class Timeslice;
}  // namespace fles

namespace cbm::algo
{
  class Options;

  struct RecoResults {
    std::vector<DigiEvent> events;
  };

  class Reco : SubChain {
  public:
    Reco();
    ~Reco();

    Reco(const Reco&) = delete;
    Reco& operator=(const Reco&) = delete;
    Reco(Reco&&)                 = delete;
    Reco& operator=(Reco&&) = delete;

    void Init(const Options&);
    RecoResults Run(const fles::Timeslice&);
    void Finalize();
    void PrintTimings(xpu::timings&);

  private:
    bool fInitialized = false;
    ChainContext fContext;
    xpu::timings fTimesliceTimesAcc;

    size_t prevTsId = 0;

    // STS
    UnpackChain fUnpack;
    sts::HitfinderChain fStsHitFinder;

    std::unique_ptr<evbuild::EventbuildChain> fEventBuild;

    void Validate(const Options& opts);

    void QueueUnpackerMetrics(const fles::Timeslice&, const UnpackMonitorData&, const DigiData&);
    void QueueStsRecoMetrics(const sts::HitfinderMonitor&);
  };
}  // namespace cbm::algo

#endif
